<?php

namespace Drupal\media_contextual_crop\PathProcessor;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\StreamWrapper\StreamWrapperManagerInterface;
use Drupal\crop\CropInterface;
use Drupal\image\PathProcessor\PathProcessorImageStyles as BaseFromImage;
use Symfony\Component\HttpFoundation\Request;

/**
 * Defines a path processor to rewrite contextualized image styles URLs.
 *
 * @see \Drupal\image\PathProcessor\PathProcessorImageStyles;
 */
class PathProcessorImageStyles extends BaseFromImage {

  /**
   * The Get EntityTypeManagerInterface.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a new PathProcessorImageStyles object.
   *
   * @param \Drupal\Core\StreamWrapper\StreamWrapperManagerInterface $stream_wrapper_manager
   *   The stream wrapper manager service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Get entity_type_manager.
   */
  public function __construct(StreamWrapperManagerInterface $stream_wrapper_manager, EntityTypeManagerInterface $entity_type_manager) {
    $this->streamWrapperManager = $stream_wrapper_manager;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function processInbound($path, Request $request) {

    $directory_path = $this->streamWrapperManager->getViaScheme('public')->getDirectoryPath();
    if (strpos($path, '/' . $directory_path . '/contextual/') === 0) {
      $path_prefix = '/' . $directory_path . '/contextual/';
    }
    // Check if the string '/system/files/styles/' exists inside the path,
    // that means we have a case of private file's image style.
    elseif (str_contains($path, '/system/files/contextual/')) {
      $path_prefix = '/system/files/contextual/';
      $path = substr($path, strpos($path, $path_prefix), strlen($path));
    }
    else {
      return $path;
    }

    // Strip out path prefix.
    $rest = preg_replace('|^' . preg_quote($path_prefix, '|') . '|', '', $path);

    // Get the image style, scheme and path.
    if (substr_count($rest, '/') >= 4) {
      [$style_separator, $image_style, $context, $scheme, $file] = explode('/', $rest, 5);

      // Set the file as query parameter.
      // Needed for private files.
      $request->query->set('file', $file);
      return $path_prefix . $style_separator . '/' . $image_style . '/' . $context . '/' . $scheme;
    }
    // Set file query parameter if its still missing from the crop ID.
    if (substr_count($rest, '/') == 3 && !$request->query->get('file')) {
      [$style_separator, $image_style, $crop_id, $scheme] = explode('/', $rest, 4);
      $crop = $this->entityTypeManager->getStorage('crop')->load($crop_id);
      if ($crop instanceof CropInterface) {
        $request->query->set('file', $crop->uri->value);
      }
    }
    return $path;
  }

}
